#!/usr/bin/env python
import sys
import numpy as np
import math
from scipy import ndimage
from netCDF4 import Dataset

r=6371229. # radius of the earth (m) 
def pxlarea(lati,latr,lonr):
  # sphere surface area: S=2pi*r*r*(1-sin(theta))
  return abs(2.*math.pi*r**2*(math.sin(np.float64(lati)/180.*math.pi)-math.sin((np.float64(lati)-np.float64(latr))/180.*math.pi))*np.float64(lonr)/360.)

def latdist(lati,latj):
  arc=np.cos(np.float64(lati)/180.*np.pi)*np.cos(np.float64(latj)/180.*np.pi)+np.sin(np.float64(lati)/180.*np.pi)*np.sin(np.float64(latj)/180.*np.pi)
  arc=np.minimum(1.,arc)
  arc=np.maximum(-1.,arc)
  return np.where(lati>latj,-r*np.arccos(arc),r*np.arccos(arc))

def londist(lat,loni,lonj):
  arc=np.cos(np.float64(lat)/180.*np.pi)**2*np.cos((np.float64(loni)-np.float64(lonj))/180.*np.pi)+np.sin(np.float64(lat)/180.*np.pi)**2
  arc=np.minimum(1.,arc)
  arc=np.maximum(-1.,arc)
  return np.where(loni>lonj,-r*np.arccos(arc),r*np.arccos(arc))

ithres1=int(float(sys.argv[1])*1000)
ithres2=int(float(sys.argv[2])*1000)

path='/home/satellites8/ylwang/PMIF/Targets/'
emisnc1=Dataset('/home/satellites8/ylwang/inventory/ODIAC/odiac2017_1km_surf_excl_intl_2016.nc','r')
emis1=emisnc1.variables['emission']
emisnc2=Dataset('/home/satellites8/ylwang/inventory/ODIAC/odiac2017_1km_point_excl_intl_2016.nc','r')
emis2=emisnc2.variables['emission']

clump=np.zeros((18000,43200))-32768
center={}
bud=[]
bud0=[]
latr=[]
lonr=[]
latrkm=[]
lonrkm=[]
npts=[]
area=[]
cent=[]
nps1=1
nps2=1
for ii in range(0,136*120,5*120):
  for jj in range(0,346*120,5*120):
    print 90-ii//120,-180+jj//120
    llat=np.arange(90-ii//120-1./240,60-ii//120,-1./120.).reshape(-1,1)
    llon=np.arange(-180+ii//120+1./240,-150+ii//120,1./120.).reshape(1,-1)
    AREA=np.array([pxlarea(90-ii//120-i/120.,1./120,1./120) for i in range(1800)]).reshape(-1,1)
    field=emis1[ii:ii+15*120,jj:jj+15*120]+emis2[ii:ii+15*120,jj:jj+15*120]
    label=np.load(path+'Targets_RW_morning_lat%i_lon%i_thres%i_%i.npy'%(90-ii//120,-180+jj//120,ithres1,ithres2))
    labels=np.unique(label[label>=0])
    center=ndimage.measurements.center_of_mass(field,label,labels)
    for il in range(len(labels)):
      if labels[il]>10000 and labels[il]<100000:
        clump[ii:ii+15*120,jj:jj+15*120][label==labels[il]]=nps1+10000
        cent.append([nps1+10000,90.-ii//120-center[il][0]/120.,-180.+jj/120+center[il][1]/120.])
        nps1+=1
      elif labels[il]>1e5: 
        clump[ii:ii+15*120,jj:jj+15*120][label==labels[il]]=nps2+100000
        cent.append([nps2+100000,90.-ii//120-center[il][0]/120.,-180.+jj/120+center[il][1]/120.])
        nps2+=1
      else:
        if len(cent)>0 and (labels[il] in np.array(cent)[:,0]):continue
        clump[ii:ii+15*120,jj:jj+15*120][label==labels[il]]=labels[il]
        cent.append([labels[il],90.-ii//120-center[il][0]/120.,-180.+jj/120+center[il][1]/120.])
      bud.append(np.sum(field[label==labels[il]]))
      pos=np.argwhere(label==labels[il])
      latr.append(np.max(pos[:,0])-np.min(pos[:,0]))
      lonr.append(np.max(pos[:,1])-np.min(pos[:,1]))
      latrkm.append(latdist(llat[int(round(np.max(pos[:,0]))),0],llat[int(round(np.min(pos[:,0]))),0]))
      lonrkm.append(londist(llat[int(round(np.median(pos[:,0]))),0],llon[0,int(round(np.max(pos[:,1])))],llon[0,int(round(np.min(pos[:,1])))]))
      npts.append(len(pos))
      area.append(np.sum(AREA*(label==labels[il])))

arr=np.hstack((cent,np.reshape(bud,(-1,1)),np.reshape(latr,(-1,1)),np.reshape(lonr,(-1,1)),np.reshape(latrkm,(-1,1)),np.reshape(lonrkm,(-1,1)),np.reshape(npts,(-1,1)),np.reshape(area,(-1,1))))
arr.dump(path+'Targets_center_RW_morning_thres%i_%i.npy'%(ithres1,ithres2))

resultnc=Dataset(path+'Targets_RW_morning_thres%i_%i.nc'%(ithres1,ithres2),'w')
resultnc.createDimension('latitude',18000)
resultnc.createDimension('longitude',43200)
var=resultnc.createVariable('latitude','d',('latitude',))
var[:]=np.arange(90.-1./240.,-60.,-1./120.)
var.units='degrees_north'
var=resultnc.createVariable('longitude','d',('longitude',))
var[:]=np.arange(-180.+1./240.,180.,1./120.)
var.units='degrees_east'
var=resultnc.createVariable('clump','i',('latitude','longitude'))
var[:]=clump[:]
resultnc.close()
