#!/usr/bin/env python
import sys
import numpy as np
from netCDF4 import Dataset
from scipy import ndimage
from skimage import segmentation,exposure,morphology

""" 
The globe is split into 15 deg x 15 deg windows to reduce memory cost.
Within each window, only the clumps whose centers are in the central 5 deg x 5 deg area
  are retained to avoid edge effect. 
Consecutive windows overlap each other by 5 deg.
"""
def clump(starti,endi,startj,endj,ithres1,ithres2):
  path='/home/satellites8/ylwang/PMIF/Targets/'

  # emission rates for area emissions (30 arcsecond x 30 arcsecond)
  emisratenc1=Dataset('/home/satellites8/ylwang/inventory/ODIAC/odiac2017_1km_surf_excl_intl_2016_emisrate.nc','r')
  emisrate1=emisratenc1.variables['emission']
  # emission rates for point sources (30 arcsecond x 30 arcsecond)
  emisratenc2=Dataset('/home/satellites8/ylwang/inventory/ODIAC/odiac2017_1km_point_excl_intl_2016_emisrate.nc','r')
  emisrate2=emisratenc2.variables['emission']
  
  # ESRI-urban area (30 arcsecond x 30 arcsecond)
  uanc=Dataset('/home/satellites8/ylwang/PMIF/Urban.nc','r')
  ua=uanc.variables['Urban']
  
  # Scaling factors for morning emissions from TIMES (0.25 deg x 0.25 deg)
  msf=np.load('/home/satellites8/ylwang/inventory/ODIAC/MorningHourlyScalingFactor.npy')
  
  for ii in range(starti,endi,5*120):
    for jj in range(startj,endj,5*120):
      print 'processing:',ii/120,jj/120
      beta=13
      # resample the scaling factors for morning emissions at 30 arcsec resolution (nearest neighbourhood)
      msf_win=msf[ii//30:ii//30+15*4,jj//30:jj//30+15*4]
      msf_win=np.repeat(msf_win,30,0)
      msf_win=np.repeat(msf_win,30,1)
      # select emission field in the window, unit conversion: gC m-2 yr-1 -> gC m-2 hr-1
      field1=emisrate1[ii:ii+15*120,jj:jj+15*120]*msf_win/24./366
      field2=emisrate2[ii:ii+15*120,jj:jj+15*120]*msf_win/24./366
      # select ESRI urban map in the window, non-urban area is set to -1
      urban=ua[ii:ii+15*120,jj:jj+15*120].filled(-1)+1
      label_fin=np.zeros((1800,1800),dtype=np.int)-1
      # A) power plants; labelled from 100001
      pp=field2>=ithres1
      label_fin[pp>0]=np.arange(1,np.sum(pp)+1)+1e5
      # B) Area clumps
      # Step 1) select pixels with emission rates > threshold 2
      mask2=(field1>=ithres2)
      # remove small holes of size <25 pixels
      mask2=morphology.remove_small_holes(mask2,25)*np.logical_not(pp)
      # End of Step 1)
      # Step 2) ESRI cores
      # select pixels with emission rates > threshold-1
      mask0=(field1>=ithres1)
      # remove small holes of size <25 pixels
      mask0=morphology.remove_small_holes(mask0,25)
      mask1=mask0*(urban>0)
      # ESRI cores with the original label from ESRI urban map, otherwise 0
      label1=np.where(mask1,urban,0)
      labels1=np.unique(label1[label1>0])
      for ll in labels1:
        label1[urban==ll]=ll
      # Step 3)
      # non-ESRI cores
      label2=ndimage.label(mask0)[0]
      labels2=np.unique(label2[label2>0])
      # if no pixel in the window has emission rate > threshold-1 -> no clump
      if not len(labels2): 
        np.save(path+'Targets_RW_morning_lat%i_lon%i_thres%i_%i.npy'%(90-ii//120,-180+jj//120,ithres1*1e3,ithres2*1e3),np.zeros((1800,1800)).astype('int32')-1)
        continue
      # if some pixels have emission rate > threshold-1, start clumping
      # to check each core, if it overlaps with ESRI-urban area, remove it (set to 0)
      for ll in labels2:
        if np.sum(mask1[label2==ll]):label2[(label2==ll)]=0
      # Step 4)
      # Combine ESRI cores and non-ESRI cores
      label=np.where(label2>0,label2+label1.max(),label1)
      # RW requires the lables must be sequential integers
      label3=segmentation.relabel_sequential(label)[0]
      # Rescale emission rates
      image=exposure.rescale_intensity(np.where(field1>0,np.log(field1),np.log(field1[field1>0].min()/1e5)))
      # makers: -1: never clumped; 0: to be clumped; positive integers: cores
      markers=np.where(np.logical_not(mask2),-1,label3)
      # RW
      label4=segmentation.random_walker(image,markers,beta=beta,mode='cg_mg',tol=1e-10)
      label4[np.logical_not(mask2)]=-1
      labels4=np.unique(label4[label4>0])
      # Combine power plants and area clumps 
      label5=label_fin+label4
      labels5=np.unique(label5[label5>0])
      center=ndimage.measurements.center_of_mass(field1+field2,label5,labels5)
      # remove clumps whose centers are outside the central 5 deg x 5 deg window
      # be careful with the edge of the earth coordinates
      for ic in range(len(center)):
        if ii==0 and jj==0 and center[ic][0]<1200 and center[ic][1]<1200:continue
        elif ii==0 and jj==345*120 and center[ic][0]<1200 and center[ic][1]>600:continue
        elif ii==135*120 and jj==0 and center[ic][0]>600 and center[ic][1]<1200:continue
        elif ii==135*120 and jj==345*120 and center[ic][0]>600 and center[ic][1]>600:continue
        elif ii==0 and center[ic][0]<1200 and center[ic][1]>600 and center[ic][1]<1200:continue
        elif ii==135*120 and center[ic][0]>600 and center[ic][1]>600 and center[ic][1]<1200:continue
        elif jj==0 and center[ic][0]>600 and center[ic][0]<1200 and center[ic][1]<1200:continue
        elif jj==345*120 and center[ic][0]>600 and center[ic][0]<1200 and center[ic][1]>600:continue
        elif center[ic][0]>600 and center[ic][0]<1200 and center[ic][1]>600 and center[ic][1]<1200:continue
        label5[label5==labels5[ic]]=-1 
      label_fin=np.array(label5)
      # Relabel clumps, ESRI clumps are labelled with ESRI urban area index;
      # non-ESRI clumps are labelled from 10001
      # power plants are labelled from 100001
      labels=np.unique(label_fin[label_fin>0])
      kk=1
      for ll in labels:
        if sum(urban[label_fin==ll]>0):label_fin[label_fin==ll]=urban[(label_fin==ll)*(urban>0)][0]-1
        else:label_fin[label_fin==ll]=10000+kk;kk+=1
      np.save(path+'Targets_RW_morning_lat%i_lon%i_thres%i_%i.npy'%(90-ii//120,-180+jj//120,ithres1*1000,ithres2*1000),label_fin)
  return #field,urban,label1

if __name__=='__main__':
  ithres1=float(sys.argv[1])
  ithres2=float(sys.argv[2])
  starti=int(sys.argv[3])
  endi=int(sys.argv[4])
  startj=int(sys.argv[5])
  endj=int(sys.argv[6])
  clump(starti,endi,startj,endj,ithres1,ithres2)
